class
	AST_ENUM_DECLARATION

inherit
	AST_DECLARATION
		redefine
			make_copy
		end

create
	make_from_antlr,
	make_copy

feature {NONE}

	make_from_antlr (node: ANTLR_AST_NODE; a_parent: AST_NODE; a_source_info: SOURCE_INFO)
		require
			node /= Void and then node.type = C_kw_enum
			a_source_info /= Void
		do
			parent := a_parent

			node.children.start
			identifier := node.children.item.token_text
			orig_c_identifier := identifier.twin
			node.children.forth
			if not node.children.after then
				from
					fields := create {TWO_WAY_LIST [AST_VARIABLE_DECLARATION]}.make
				until
					node.children.after
				loop
					fields.extend (create {AST_VARIABLE_DECLARATION}.make (
							create {AST_BASIC_TYPE}.make (Ast_bt_int),
							node.children.item.children.first.token_text,
							process_expression (node.children.item.children.last, Void)
						))
					fields.last.parent := Current
					node.children.forth
				end
			end
			source_info := a_source_info
		end

feature {ANY}

	fields: LIST [AST_VARIABLE_DECLARATION]

	is_forward_declaration: BOOLEAN
		do
			Result := fields = Void
		end

	replacement_type: AST_BASIC_TYPE
		local
			l_eval: INTEGER_CONSTANT_EXPRESSION_EVALUATOR
			l_has_negative_fields: BOOLEAN
		once ("OBJECT")
			across
				fields as c
			until
				l_has_negative_fields
			loop
				create l_eval.make (c.item.initializer)
				l_has_negative_fields := l_eval.state = {INTEGER_CONSTANT_EXPRESSION_EVALUATOR}.State_integer and then l_eval.integer < 0
			end
			if l_has_negative_fields then
				create Result.make (Ast_bt_int)
			else
				create Result.make (Ast_bt_unsigned_int)
			end
		ensure
			Result /= Void and then (Result.type = Ast_bt_int or Result.type = Ast_bt_unsigned_int)
		end

	orig_c_identifier: STRING

feature {ANY} -- AST_NODE

	accept (v: AST_VISITOR)
		local
			visit_children: BOOLEAN
		do
			v.pre_visit (Current)
			visit_children := v.visit_enum_declaration (Current)
			if visit_children then
				visit_list (fields, v)
			end
			v.end_visit_enum_declaration (Current)
			v.post_visit (Current)
		end

	ast_twin: like Current
		do
			create Result.make_copy (Current)
		end

	make_copy (a_other: like Current)
		do
			Precursor (a_other)
			orig_c_identifier := a_other.orig_c_identifier -- won't be changed
			if a_other.fields /= Void then
				fields := create {TWO_WAY_LIST [AST_VARIABLE_DECLARATION]}.make
				across a_other.fields as c loop
					fields.extend (c.item.ast_twin)
					fields.last.parent := Current
				end
			end
		end

invariant
	fields /= Void implies not fields.is_empty
	orig_c_identifier /= Void and then not orig_c_identifier.is_empty

end
