class
	AST_EIFFEL_EXPRESSION

inherit
	AST_EXPRESSION
	AST_UTILITIES

create
	make,
	make_mixed,
	make_copy

feature {NONE}

	make (a_code: STRING; a_type_binding: AST_TYPE)
		require
			a_code /= Void
			a_type_binding /= Void
		do
			mixed := create {TWO_WAY_LIST [ANY]}.make
			mixed.extend (a_code)
			type_binding := a_type_binding
		end

	make_mixed (a_mixed: ARRAY [ANY]; a_type_binding: AST_TYPE)
		require
			a_mixed /= Void and then not a_mixed.is_empty and then across a_mixed as c all
					attached {STRING} c.item or (attached {AST_NODE} c.item as tmp and then tmp.parent = Void)
				end
			a_type_binding /= Void
		do
			mixed := create {TWO_WAY_LIST [ANY]}.make
			across a_mixed as c loop
				if attached {AST_NODE}c.item as node then
					node.parent := Current
				end
				mixed.extend (c.item)
			end
			type_binding := a_type_binding
		end

feature {ANY}

	mixed: LIST [ANY]

	type_binding: AST_TYPE

	is_string (a_index: INTEGER_32; a_string: STRING_8): BOOLEAN
		require
			a_index >= 1 and a_index <= mixed.count
			a_string /= Void
		do
			Result := attached {STRING_8} mixed.i_th (a_index) as tmp and then tmp.is_equal (a_string)
		end

	next_string (a_child: ANY): STRING_8
		require
			a_child /= Void and then mixed.has (a_child)
		local
			l_done: BOOLEAN
		do
			across
				mixed as c
			until
				l_done
			loop
				if c.item = a_child then
					c.forth
					if not c.after then
						Result ?= c.item
					end
					l_done := True
				end
			end
		end

	next_node (a_child: ANY): AST_NODE
		require
			a_child /= Void and then mixed.has (a_child)
		local
			l_done: BOOLEAN
		do
			across
				mixed as c
			until
				l_done
			loop
				if c.item = a_child then
					c.forth
					if not c.after then
						Result ?= c.item
					end
					l_done := True
				end
			end
		end

	previous_string (a_child: ANY): STRING_8
		require
			a_child /= Void and then mixed.has (a_child)
		local
			l_done: BOOLEAN
			l_previous: ANY
		do
			across
				mixed as c
			until
				l_done
			loop
				if c.item = a_child then
					Result ?= l_previous
					l_done := True
				end
				l_previous := c.item
			end
		end

	previous_node (a_child: ANY): AST_NODE
		require
			a_child /= Void and then mixed.has (a_child)
		local
			l_done: BOOLEAN
			l_previous: ANY
		do
			across
				mixed as c
			until
				l_done
			loop
				if c.item = a_child then
					Result ?= l_previous
					l_done := True
				end
				l_previous := c.item
			end
		end

	insert_before (a_child, a_new_child: ANY)
		require
			a_child /= Void and then mixed.has (a_child)
			attached {STRING} a_new_child or (attached {AST_NODE} a_new_child as tmp and then tmp.parent = Void)
		local
			l_two_way_list: TWO_WAY_LIST [ANY]
		do
			from
				l_two_way_list ?= mixed
				l_two_way_list.start
			until
				l_two_way_list.item = a_child
			loop
				l_two_way_list.forth
			end
			l_two_way_list.put_left (a_new_child)
			if attached {AST_NODE} a_new_child as tmp then
				tmp.parent := Current
			end
		ensure
			mixed.count = old mixed.count + 1
		end

feature {ANY} -- AST_NODE

	accept (v: AST_VISITOR)
		local
			visit_children: BOOLEAN
		do
			v.pre_visit (Current)
			visit_children := v.visit_eiffel_expression (Current)
			if visit_children then
				across mixed.twin as c loop
					if attached {AST_NODE} c.item as node then
						node.accept (v)
					end
				end
			end
			v.end_visit_eiffel_expression (Current)
			v.post_visit (Current)
		end

	ast_twin: like Current
		do
			create Result.make_copy (Current)
		end

	make_copy (other: like Current)
		do
			mixed := create {TWO_WAY_LIST [ANY]}.make
			across other.mixed as c loop
				if attached {AST_NODE} c.item as tmp then
					mixed.extend (tmp.ast_twin)
					({AST_NODE} #? mixed.last).parent := Current
				else
					mixed.extend (c.item.twin)
				end
			end
			type_binding := other.type_binding
		end

invariant
	mixed /= Void and then not mixed.is_empty
	across mixed as c all
			attached {STRING_8} c.item or
			(attached {AST_NODE} c.item as n and then n.parent = Current)
		end

end
