-- Can be a usage of a variable (e.g. `i' in `i * 5') or the usage of a function (e.g. in `&alphasort')

class
	AST_VARIABLE_EXPRESSION

inherit
	AST_EXPRESSION

create
	make,
	make_from_antlr,
	make_copy

feature {NONE}

	make (a_declaration_binding: AST_EIFFEL_FEATURE)
		require
			a_declaration_binding /= Void
		do
			identifier := a_declaration_binding.identifier
			declaration_binding := a_declaration_binding
		end

	make_from_antlr (node: ANTLR_AST_NODE; init_parent: AST_NODE)
		require
			node /= Void and then node.type = Ast_variable_expression
		do
			parent := init_parent
			identifier := node.children.first.token_text
		end

feature {ANY}

	identifier: STRING

feature {ANY} -- bindings

	type_binding: AST_TYPE
		do
			if attached {AST_FUNCTION_DECLARATION} declaration_binding as fd then
				Result := create {AST_FUNCTION_TYPE}.make (fd.return_type.ast_twin, fd.parameter_list.ast_twin)
			else
				Result := declaration_binding.type
			end
		end

	declaration_binding: AST_EIFFEL_FEATURE assign set_declaration_binding

	set_declaration_binding (a_declaration: AST_EIFFEL_FEATURE)
		require
			a_declaration /= Void
		do
			declaration_binding := a_declaration
		end

feature {ANY} -- AST_NODE

	accept (v: AST_VISITOR)
		local
			visit_children: BOOLEAN
		do
			v.pre_visit (Current)
			visit_children := v.visit_variable_expression (Current)
			if visit_children then
				-- no children
			end
			v.end_visit_variable_expression (Current)
			v.post_visit (Current)
		end

	ast_twin: like Current
		do
			create Result.make_copy (Current)
		end

	make_copy (other: like Current)
		do
			identifier := other.identifier.twin
			declaration_binding := other.declaration_binding
		end

invariant
	identifier /= Void and then not identifier.is_empty;
	(Bindings_active.item and not is_in_orig_c_expression) implies declaration_binding /= Void

end
