class
	IDENTIFIER_TRANSFORMER

inherit
	TRANSFORMATION_STEP
	NAME_UTILITIES

feature {ANY} -- TRANSFORMATION_STEP

	process_impl: BOOLEAN
		local
			l_namespace: HASH_SET [STRING_8]
		do
			create l_namespace.make (16)
			across transformer.tree.declarations as c loop
				if attached {AST_STRUCT_DECLARATION} c.item as sd then
					sd.identifier.make_from_string (remove_cil_anonymous_decoration (sd.identifier))
					adapt_identifier (sd.identifier, l_namespace)
					sd.identifier.to_upper

					handle_class (sd)
				elseif attached {AST_ENUM_DECLARATION} c.item as ed then
					ed.identifier.make_from_string (remove_cil_anonymous_decoration (ed.identifier))
					adapt_identifier (ed.identifier, l_namespace)
					ed.identifier.to_upper

					handle_enum_class (ed)
				end
			end

			-- project classes inherit from multiple system classes;
			-- make sure the feature names are unique there too
			create l_namespace.make (16)
			across transformer.tree.declarations as c loop
				if
					attached {AST_EIFFEL_FEATURE} c.item as ef and then
					ef.source_info.is_system_source
				then
					adapt_identifier (ef.identifier, l_namespace)
				end
			end

			Result := True
		end

	description: STRING
		do
			Result := "Transforming names of fields, variables and functions"
		end

feature {NONE}

	handle_class (a_node: AST_STRUCT_DECLARATION)
		local
			l_namespace: HASH_SET [STRING_8]
		do
			create l_namespace.make (16)
			across a_node.oo_superclasses as c loop
				across c.item.fields as c2 loop
					if attached c2.item.identifier as identifier then
						l_namespace.extend (identifier)
					end
				end
				across c.item.oo_features as c2 loop
					l_namespace.extend (c2.item.identifier)
				end
				across c.item.oo_static_features as c2 loop
					l_namespace.extend (c2.item.identifier)
				end
			end
			across a_node.oo_static_superclasses as c loop
				across c.item.fields as c2 loop
					if attached c2.item.identifier as identifier then
						l_namespace.extend (identifier)
					end
				end
				across c.item.oo_features as c2 loop
					l_namespace.extend (c2.item.identifier)
				end
				across c.item.oo_static_features as c2 loop
					l_namespace.extend (c2.item.identifier)
				end
			end

			across a_node.fields as c loop
				if attached c.item.identifier as identifier then
					adapt_identifier (c.item.identifier, l_namespace)
				end
			end
			across a_node.oo_features as c loop
				adapt_identifier (c.item.identifier, l_namespace)
			end
			across a_node.oo_static_features as c loop
				adapt_identifier (c.item.identifier, l_namespace)
			end

			across a_node.oo_features as c loop
				if attached {AST_FUNCTION_DECLARATION} c.item as fd then
					handle_routine (fd, l_namespace)
				end
			end
			across a_node.oo_static_features as c loop
				if attached {AST_FUNCTION_DECLARATION} c.item as fd then
					handle_routine (fd, l_namespace)
				end
			end
		end

	handle_routine (a_node: AST_FUNCTION_DECLARATION; a_class_namespace: HASH_SET [STRING_8])
		local
			l_namespace: HASH_SET [STRING_8]
		do
			create l_namespace.make (16)
			across a_class_namespace as c loop
				l_namespace.extend (c.item)
			end

			across a_node.parameter_list.parameters as c loop
				if attached {AST_VARIABLE_DECLARATION} c.item as vd then
					vd.identifier.prepend ("a_")
					adapt_identifier (vd.identifier, l_namespace)
				end
			end
			if a_node.locals /= Void then
				across a_node.locals as c loop
					c.item.identifier.prepend ("l_")
					adapt_identifier (c.item.identifier, l_namespace)
				end
			end
		end

	handle_enum_class (a_node: AST_ENUM_DECLARATION)
		local
			l_namespace: HASH_SET [STRING_8]
		do
			create l_namespace.make (a_node.fields.count)
			across a_node.fields as c loop
				adapt_identifier (c.item.identifier, l_namespace)
				c.item.identifier[1] := c.item.identifier[1].upper
			end
		end

feature {NONE}

	adapt_identifier (a_identifier: STRING_8; a_existing_identifiers: HASH_SET [STRING_8])
			-- Changes a C identifier to an Eiffel identifier.
		require
			a_identifier /= Void and then not a_identifier.is_empty
			a_existing_identifiers /= Void
		local
			l_s: STRING_8
			l_i: INTEGER_32
		do
			a_identifier.make_from_string (pretty_print_underscores (a_identifier))

			-- fix camel case
			across a_identifier as c loop
				if
					c.target_index > 1 and then
					a_identifier[c.target_index] >= 'A' and then a_identifier[c.target_index] <= 'Z' and then
					a_identifier[c.target_index - 1] >= 'a' and then a_identifier[c.target_index - 1] <= 'z'
				then
					a_identifier.insert_character ('_', c.target_index)
					c.forth
				end
			end
			a_identifier.to_lower

			-- handle duplication and keywords
			from
				l_s := a_identifier
				l_i := 1
			until
				not a_existing_identifiers.has (l_s) and
				not eiffel_keywords.has (l_s)
			loop
				l_i := l_i + 1
				l_s := a_identifier + l_i.out
			end
			if l_i > 1 then
				a_identifier.append_integer (l_i)
			end

			a_existing_identifiers.extend (a_identifier.twin) -- twin it; might be changed
		end

	eiffel_keywords: HASH_SET [STRING_8]
		once ("PROCESS")
			create Result.make (100)
			Result.extend ("agent")
			Result.extend ("alias")
			Result.extend ("all")
			Result.extend ("and")
			Result.extend ("as")
			Result.extend ("assign")
			Result.extend ("attribute")
			Result.extend ("bit")
			Result.extend ("check")
			Result.extend ("class")
			Result.extend ("convert")
			Result.extend ("copy")
			Result.extend ("create")
			Result.extend ("current")
			Result.extend ("debug")
			Result.extend ("deferred")
			Result.extend ("do")
			Result.extend ("do_nothing")
			Result.extend ("else")
			Result.extend ("elseif")
			Result.extend ("end")
			Result.extend ("ensure")
			Result.extend ("equal")
			Result.extend ("expanded")
			Result.extend ("export")
			Result.extend ("external")
			Result.extend ("false")
			Result.extend ("feature")
			Result.extend ("from")
			Result.extend ("frozen")
			Result.extend ("if")
			Result.extend ("implies")
			Result.extend ("inherit")
			Result.extend ("inspect")
			Result.extend ("invariant")
			Result.extend ("like")
			Result.extend ("local")
			Result.extend ("loop")
			Result.extend ("not")
			Result.extend ("note")
			Result.extend ("obsolete")
			Result.extend ("old")
			Result.extend ("once")
			Result.extend ("only")
			Result.extend ("or")
			Result.extend ("out")
			Result.extend ("precursor")
			Result.extend ("prefix")
			Result.extend ("print")
			Result.extend ("redefine")
			Result.extend ("rename")
			Result.extend ("require")
			Result.extend ("rescue")
			Result.extend ("result")
			Result.extend ("retry")
			Result.extend ("select")
			Result.extend ("separate")
			Result.extend ("some")
			Result.extend ("then")
			Result.extend ("true")
			Result.extend ("tuple")
			Result.extend ("undefine")
			Result.extend ("unique")
			Result.extend ("until")
			Result.extend ("variant")
			Result.extend ("void")
			Result.extend ("when")
			Result.extend ("xor")

			-- reserved for helpers
			Result.extend ("l_aaf")
			Result.extend ("l_pos")
		end

end
