class
	VARIABLE_ADDRESS_TRANSFORMER

inherit
	TRANSFORMATION_STEP
	ROUTINE_PARAMETER_UTILITIES

feature {ANY} -- TRANSFORMATION_STEP

	process_impl: BOOLEAN
		local
			l_visitor: AST_AGENT_VISITOR
		do
			-- collect all variables
			create l_visitor
			l_visitor.agents.end_visit_unary_expression := agent collect_variable_declarations
			transformer.tree.accept (l_visitor)

			-- change the variables
			create l_visitor
			l_visitor.agents.end_visit_variable_declaration := agent adapt_variable_declarations
			l_visitor.agents.end_visit_variable_expression := agent adapt_variable_expressions
			transformer.tree.accept (l_visitor)

			Result := True
		end

	description: STRING
		do
			Result := "Adapting variables of which the address is taken"
		end

feature {NONE}

	collect_variable_declarations (a_node: AST_UNARY_EXPRESSION)
		do
			if
				-- &a
				a_node.type = C_op_bitand and then
				attached {AST_VARIABLE_EXPRESSION} a_node.expression as ot_variable_expression and then
				attached {AST_VARIABLE_DECLARATION} ot_variable_expression.declaration_binding as ot_variable_declaration
			then
				ot_variable_declaration.changed_to_array := True
			elseif
				-- &a.b.c
				a_node.type = C_op_bitand and then
				attached {AST_POSTFIX_EXPRESSION} a_node.expression as ot_postfix_expression and then
				attached {AST_VARIABLE_EXPRESSION} ot_postfix_expression.base_expression as ot_variable_expression and then
				attached {AST_VARIABLE_DECLARATION} ot_variable_expression.declaration_binding as ot_variable_declaration and then
				attached {AST_STRUCT_TYPE} ot_variable_declaration.type
			then
				ot_variable_declaration.changed_to_array := True
			end
		end

	adapt_variable_declarations (a_node: AST_VARIABLE_DECLARATION)
		do
			if a_node.changed_to_array then
				if attached {AST_PARAMETER_LIST} a_node.parent then
					-- moves it to a local; will be visited later
					a_node.changed_to_array := False
					change_to_local (a_node)
					a_node.changed_to_array := True
				else
					replace_node (a_node.type, create {AST_ARRAY_TYPE}.make (
							a_node.type.ast_twin,
							create {AST_DECIMAL_CONSTANT}.make (1, create {AST_BASIC_TYPE}.make (Ast_bt_int))
						))
					if a_node.initializer /= Void then
						replace_node (a_node.initializer, create {AST_INITIALIZER}.make (
								<<a_node.initializer.ast_twin>>,
								a_node.type
							))
					end
				end
			end
		end

	adapt_variable_expressions (a_node: AST_VARIABLE_EXPRESSION)
		do
			if
				attached {AST_VARIABLE_DECLARATION} a_node.declaration_binding as ot_variable_declaration and then
				ot_variable_declaration.changed_to_array
			then
				if
					attached {AST_UNARY_EXPRESSION} a_node.parent as ot_unary_expression and then
					ot_unary_expression.type = C_op_bitand
				then
					-- `&a' -> `a'
					replace_node (ot_unary_expression, a_node.ast_twin)
				else
					-- `a' -> `*a'
					replace_node (a_node, create {AST_UNARY_EXPRESSION}.make (C_op_mult, a_node.ast_twin))
				end
			end
		end

end
