class
	NAME_UTILITIES

inherit
	ANTLR_AST_TYPE_CONSTANTS

feature {ANY}

	eiffel_file_name (c_name: STRING_8): STRING_8
			-- `c_name' can be
			-- o a file name (e.g. "micro_httpd.cil.adapted.c") or s
			-- o a struct/union name (e.g. "_IO_FILE")
			-- returns e.g. "io_file.e" for "_IO_FILE"
		require
			c_name /= Void and then not c_name.is_empty
		do
			Result := eiffel_class_name (c_name).as_lower + ".e"
		end

	eiffel_class_name (c_name: STRING_8): STRING_8
			-- `c_name' can be
			-- o a file name (e.g. "micro_httpd.cil.adapted.c") or
			-- o a struct/union name (e.g. "_IO_FILE")
			-- returns e.g. "MICRO_HTTPD" for "micro_httpd.cil.adapted.c"
		require
			c_name /= Void and then not c_name.is_empty
		do
			create Result.make_from_string (c_name)
			from until not Result.starts_with ("_") loop
				Result := Result.substring (2, Result.count)
			end
			from until not Result.starts_with ("../") loop
				Result := Result.substring (4, Result.count)
			end
			if Result.has ('.') then
				Result := Result.substring (1, Result.index_of ('.', 1) - 1)
			end
			Result.replace_substring_all ("-", "_")
			Result.replace_substring_all ("/", "_")
			Result.to_upper

			if Result[1].is_digit then
				Result := "C2EIF_" + Result
			end
		end

	c_header_file (node: AST_TRANSLATION_UNIT): STRING_8
		require
			node /= Void
		do
			Result := eiffel_class_name (node.source_file_name).as_lower + ".h"
		end

	pretty_print_underscores (a_identifier: STRING_8): STRING_8
			-- remove leading, trailing and repeated underscores
		do
			create Result.make (a_identifier.count)
			across a_identifier as c loop
				if
					not (c.item = '_' and
					(
						Result.is_empty or -- leading
						c.target_index = a_identifier.count or -- trailing
						a_identifier[c.target_index + 1] = '_' -- repeated
					))
				then
					Result.append_character (c.item)
				end
			end
		end

	remove_cil_anonymous_decoration (a_identifier: STRING_8): STRING_8
		local
			l_prefix: STRING_8 -- "", PD_, SD_, PC_, SC_
			l_index: INTEGER_32
		do
			if
				a_identifier.starts_with ("PD_") or a_identifier.starts_with ("SD_") or
				a_identifier.starts_with ("PC_") or a_identifier.starts_with ("SC_")
			then
				l_prefix := a_identifier.substring (1, 3)
				l_index := 4
			else
				l_prefix := ""
				l_index := 1
			end
			if
				starts_with_at (a_identifier, "__anonstruct_", l_index) or
				starts_with_at (a_identifier, "__anonunion_", l_index) or
				starts_with_at (a_identifier, "__anonenum_", l_index)
			then
				-- PD___anonenum_foo_3 -> PD_foo
				Result := l_prefix + a_identifier.substring (
						a_identifier.index_of ('_', 11) + 1,
						a_identifier.last_index_of ('_', a_identifier.count - 1) - 1
					)
				if Result.is_empty or (not l_prefix.is_empty and Result.count = 3) then
					Result.append ("ANONYMOUS")
				end
			else
				Result := a_identifier
			end
		end

feature {ANY}

	effective_type (a_type: AST_TYPE): STRING_8
		require
			(attached {AST_POINTER_TYPE} a_type as pt and then not attached {AST_FUNCTION_TYPE} pt.type) or attached {AST_ARRAY_TYPE} a_type
		local
			l_main_type: STRING_8
			l_sub_type: AST_TYPE
		do
			if attached {AST_POINTER_TYPE} a_type as pt then
				l_main_type := "POINTER"
				l_sub_type := pt.type
			elseif attached {AST_ARRAY_TYPE} a_type as at then
				l_main_type := "ARRAY"
				l_sub_type := at.type
			end

			if attached {AST_POINTER_TYPE} l_sub_type as pt then
				if attached {AST_FUNCTION_TYPE} pt.type then
					Result := "CE_ROUTINE_" + l_main_type
				else
					Result := "CE_POINTER_" + l_main_type + " [" + pt.type.eiffel_code + "]"
				end
			elseif attached {AST_ARRAY_TYPE} l_sub_type as at then
				Result := "CE_ARRAY_" + l_main_type + " [" + at.type.eiffel_code + "]"
			elseif attached {AST_STRUCT_TYPE} l_sub_type then
				Result := "CE_CLASS_" + l_main_type + " [" + l_sub_type.eiffel_code + "]"
			elseif attached {AST_FUNCTION_TYPE} l_sub_type then
				Result := "CE_ROUTINE"
			elseif attached {AST_BASIC_TYPE} l_sub_type as bt and then bt.type = Ast_bt_void then
				Result := "CE_VOID_POINTER"
			elseif attached {AST_BASIC_TYPE} l_sub_type as bt and then bt.type = Ast_bt_builtin_va_list then
				Result := "CE_VA_LIST_" + l_main_type
			else
				Result := "CE_" + l_sub_type.eiffel_code + "_" + l_main_type
			end
		ensure
			Result /= Void
		end

feature {NONE}

	multiply (s: STRING; count: INTEGER): STRING
		require
			count >= 0
		do
			if count = 0 then
				Result := ""
			else
				Result := s
				Result.multiply (count)
			end
		ensure
			Result /= Void
		end

	starts_with_at (a_string, a_substring: STRING_8; a_index: INTEGER_32): BOOLEAN
			-- Does `a_string' start with `a_substring' at index `a_index'?
		do
			if a_index = 1 then
				Result := a_string.starts_with (a_substring)
			else
				-- I'm a baaad person!
				Result := a_string.substring (a_index, a_string.count).starts_with (a_substring)
			end
		end

end
